/*
   Teensy Color Organ
     File: BaseScreen.h - Purpose: Parent C++ Class for all Screens

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern XPT2046_Touchscreen touch;

#ifndef BASESCREEN
#define BASESCREEN

// Touch screen calibration data
// Touch map
struct TS_MAP {
  int16_t xTop;
  int16_t xBottom;
  int16_t yLeft;
  int16_t yRight;
};

TS_MAP tsMap[5] = {

  { 200, 3700, 325, 3670 }, // Touch to pixel calibration data
  {   0,  319,   0,  239 }, // Rotation 0 mapping
  { 319,    0,   0,  239 }, // Rotation 1 mapping
  { 319,    0, 239,    0 }, // Rotation 2 mapping
  {   0,  319, 239,    0 }  // Rotation 3 mapping
};

class BaseScreen {

  public:

    // Pure virtual functions every subclass must implement
    virtual void init(void) = 0;
    virtual enum STATES run(void) = 0;

  protected:

    // Read any touch events
    bool readTouch(void) {

      // Was the screen touched ?
      if (touch.touched()) {

        // Yes so read the touch event
        TS_Point p = touch.getPoint();

        // Calibrate and map touch values to screen
        touchX = map(p.x, tsMap[0].xTop,  tsMap[0].xBottom, tsMap[SCREEN_ROTATION].xTop,  tsMap[SCREEN_ROTATION].xBottom);
        touchY = map(p.y, tsMap[0].yLeft, tsMap[0].yRight,  tsMap[SCREEN_ROTATION].yLeft, tsMap[SCREEN_ROTATION].yRight);

        // Wait for end of the touch event before continuing
        while (touch.touched()) {
          touch.getPoint();
        }
        // Indicate touch event occurred
        return true;

      } else  {
        touchX = -1;
        touchY = -1;

        // Indicate no touch event occurred
        return false;
      }
    }

    // Convert integer value to char string
    char *convertIntToString(int i) {
      char buffer[10];

      return itoa(i, buffer, 10);
    }

    // Protected data
    int touchX, touchY;
};

#endif

