/*
   Teensy Color Organ
     File: Colors.h - Purpose: Color creation functions

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern Adafruit_NeoPixel ledStrip;


#ifndef COLORS
#define COLORS

#if (USE_GAMMA_CORRECTION == true)
const byte gammaTable[] = {
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,
  1,  1,  1,  1,  1,  1,  1,  1,  1,  2,  2,  2,  2,  2,  2,  2,
  2,  3,  3,  3,  3,  3,  3,  3,  4,  4,  4,  4,  4,  5,  5,  5,
  5,  6,  6,  6,  6,  7,  7,  7,  7,  8,  8,  8,  9,  9,  9, 10,
  10, 10, 11, 11, 11, 12, 12, 13, 13, 13, 14, 14, 15, 15, 16, 16,
  17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 24, 24, 25,
  25, 26, 27, 27, 28, 29, 29, 30, 31, 32, 32, 33, 34, 35, 35, 36,
  37, 38, 39, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 50,
  51, 52, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 66, 67, 68,
  69, 70, 72, 73, 74, 75, 77, 78, 79, 81, 82, 83, 85, 86, 87, 89,
  90, 92, 93, 95, 96, 98, 99, 101, 102, 104, 105, 107, 109, 110, 112, 114,
  115, 117, 119, 120, 122, 124, 126, 127, 129, 131, 133, 135, 137, 138, 140, 142,
  144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 167, 169, 171, 173, 175,
  177, 180, 182, 184, 186, 189, 191, 193, 196, 198, 200, 203, 205, 208, 210, 213,
  215, 218, 220, 223, 225, 228, 231, 233, 236, 239, 241, 244, 247, 249, 252, 255
};
#endif

// Convert HSV color to RGB color
void hsvToRGB(byte h, byte s, byte v, byte *r, byte *g, byte *b) {

  byte region, fpart, p, q, t;

  if (s == 0) {
    /* color is grayscale */
    *r = *g = *b = v;
    return;
  }

  /* make hue 0-5 */
  region = h / 43;
  /* find remainder part, make it from 0-255 */
  fpart = (h - (region * 43)) * 6;

  /* calculate temp vars, doing integer multiplication */
  p = (v * (255 - s)) >> 8;
  q = (v * (255 - ((s * fpart) >> 8))) >> 8;
  t = (v * (255 - ((s * (255 - fpart)) >> 8))) >> 8;

  /* assign temp vars based on color cone region */
  switch (region) {
    case 0:
      *r = v; *g = t; *b = p; break;
    case 1:
      *r = q; *g = v; *b = p; break;
    case 2:
      *r = p; *g = v; *b = t; break;
    case 3:
      *r = p; *g = q; *b = v; break;
    case 4:
      *r = t; *g = p; *b = v; break;
    default:
      *r = v; *g = p; *b = q; break;
  }
}

// Create a fully saturated 16 bit color from hue and value
// This is used for creating colors used on the LCD display
// Hue is 0 .. 255 - Value is 0 .. 255
uint16_t create16BitColor(byte hue, byte value) {
  byte r, g, b;

  if (value == 0) {
    return 0;
  }

  hsvToRGB(hue, 255, value, &r, &g, &b);
  return (((uint16_t) r &  0xF8) << 8) | (((uint16_t) g  & 0xFC) << 3) | ((uint16_t) b >> 3);
}

// Create a fully saturated 24 bit color from hue and value
// This is used for creating colors used on the LED strip
// Hue is 0 .. 255 - Value is 0 .. 255
uint32_t create24BitColor(byte hue, byte value) {
  byte r, g, b;

  if (value == 0) {
    return 0;
  }

  hsvToRGB(hue, 255, value, &r, &g, &b);

#if (USE_GAMMA_CORRECTION == true)
  return ledStrip.Color(gammaTable[r], gammaTable[g], gammaTable[b]);
#else
  return ledStrip.Color(r, g, b);
#endif
}

#endif

