/*
   Teensy Color Organ
     File: Screen_Config.h - Purpose: Screen for selecting type of configuration to do

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#ifndef CONFIGSCREEN
#define CONFIGSCREEN

// Instantiate the buttons
GUI_Button miscButton(    14,  52, 88, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Misc");
GUI_Button densityButton(116,  52, 88, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Density");
GUI_Button filesButton(  218,  52, 88, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Files");

GUI_Button backButton(   116, 120, 88, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Back");

class ConfigScreen : public BaseScreen {

  public:
    // Initialize the screen
    void init(void) {
      // Clear the screen and draw the border
      drawBorder();

      lcd.setTextSize(2);

      // Make the buttons visible
      miscButton.paint();
      densityButton.paint();
      filesButton.paint();
      backButton.paint();

      lcd.setTextSize(1);
      lcd.setTextColor(ILI9341_WHITE);

      // Draw message on screen
      printCenteredString("Touch button to configure item", 0, 185, 320, 16);
    }

    // Run the screen's code
    enum STATES run(void) {

      // Has the screen been touched
      if (readTouch()) {

        // Determine which button if any was touched
        // Has the back button been touched ?
        if (backButton.touched(touchX, touchY)) {
          return MAIN_SCR_INIT_STATE;

        } else if (miscButton.touched(touchX, touchY)) {
          return MISC_SCR_INIT_STATE;

        } else if (densityButton.touched(touchX, touchY)) {
          return DENSITY_SCR_INIT_STATE;

        } else if (filesButton.touched(touchX, touchY)) {
          return FILES_SCR_INIT_STATE;
        }
      }
      return CONFIG_SCR_STATE;
    }
};

#endif

