/*
   Teensy Color Organ
     File: Screen_Density.h - Purpose: Screen and controls for setting per band LED density

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#ifndef DENSITYSCREEN
#define DENSITYSCREEN

#define MIN_DENSITY  10
#define MAX_DENSITY  90
#define DENSITY_INC   5

// Instantiate value picker
GUI_ValuePicker densityPicker(90, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE);

// Instantiate buttons
GUI_Button dBackButton(120, 160, 80, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Back");

class DensityScreen : public BaseScreen {

  public:

    // Initialize the screen
    void init(void) {

      // Clear the screen and draw the border
      drawBorder();

      // Set big font
      lcd.setTextSize(2);
      lcd.setTextColor(ILI9341_WHITE);

      // Draw screen labels
      printCenteredString("LED Density Selector in %", 0, 45, 320, 16);

      // Install default values
      densityPicker.setTextFieldText(convertIntToString(configurationData.ledDensity));

      // Make screen controls visible
      densityPicker.paint();
      dBackButton.paint();
    }

    enum STATES run(void) {

      // Has the screen been touched
      if (readTouch()) {

        // Has the back button been touched ?
        if (dBackButton.touched(touchX, touchY)) {
          // Yes return to configuration screen
          return CONFIG_SCR_INIT_STATE;
        }

        // Was the density picker incrementer touched ?
        if (densityPicker.incrementerTouched(touchX, touchY)) {
          configurationData.ledDensity += DENSITY_INC;
          if (configurationData.ledDensity > MAX_DENSITY) {
            configurationData.ledDensity = MAX_DENSITY;
          }
          ledOutputControl.assignLEDsBasedOnDensity();
        }

        // Was the density picker decrementer touched ?
        if (densityPicker.decrementerTouched(touchX, touchY)) {
          if (configurationData.ledDensity <= DENSITY_INC) {
            configurationData.ledDensity = MIN_DENSITY;
          } else {
            configurationData.ledDensity -= DENSITY_INC;
          }
          ledOutputControl.assignLEDsBasedOnDensity();
        }

        // Update value pickers text field color
        densityPicker.setTextFieldText(convertIntToString(configurationData.ledDensity));
      }
      // Stay on this screen until back button is touched
      return DENSITY_SCR_STATE;
    }
};

#endif

