/*
   Teensy Color Organ
     File: Screen_Main.h - Purpose: Main screen of color organ showing stereo VU meters

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#ifndef MAINSCREEN
#define MAINSCREEN

// Instantiate the stereo VU meters
GUI_VUField leftVUMeter(100, 35, 15, 110, ILI9341_YELLOW, ILI9341_BLACK, ILI9341_WHITE);
GUI_VUField rightVUMeter(210, 35, 15, 110, ILI9341_YELLOW, ILI9341_BLACK, ILI9341_WHITE);

class MainScreen : public BaseScreen {

  public:
    // Initialize the screen
    void init(void) {
      // Clear the screen and draw the border
      drawBorder();

      // Make the VU meters visible
      leftVUMeter.paint();
      rightVUMeter.paint();

      lcd.setTextSize(2);
      lcd.setTextColor(ILI9341_WHITE);
      printCenteredString("Left", 90, 158, 30, 16);
      printCenteredString("Right", 200, 158, 30, 16);

      lcd.setTextSize(1);
      printCenteredString("Audio", 146, 70, 30, 16);
      printCenteredString("Levels", 146, 100, 30, 16);
      printCenteredString("Touch screen for configuration", 0, 186, 320, 16);
    }

    // Run the screen's code
    enum STATES run(void) {

      // Has the screen been touched
      if (readTouch()) {
        return CONFIG_SCR_INIT_STATE;
      } else  {
        return MAIN_SCR_STATE;
      }
    }

    // Update the stereo VU meters
    void updateVUMeters(float leftLevel, float rightLevel) {
      // Update VU meters
      leftVUMeter.setValue(leftLevel);
      rightVUMeter.setValue(rightLevel);
    }
};

#endif

